# Here's an example parameter that will recieve
# a controller node through the OCI's ENC:
#
#---
#classes:
#   oci_controller:
#      is_first_master: true
#      first_master: zigo-controller-node-3.infomaniak.ch
#      first_master_ip: 192.168.100.40
#      other_masters:
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      other_masters_ip:
#         - 192.168.100.41
#         - 192.168.100.32
#      all_masters:
#         - zigo-controller-node-3.infomaniak.ch
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      all_masters_ip:
#         - 192.168.100.40
#         - 192.168.100.41
#         - 192.168.100.32
#
# This is re-used in the oci_controller class below
#
class oci::controller(
  $is_first_master          = false,
  $first_master             = undef,
  $first_master_ip          = undef,
  $other_masters            = undef,
  $other_masters_ip         = undef,
  $vip_hostname             = undef,
  $vip_ipaddr               = undef,
  $all_masters              = undef,
  $all_masters_ip           = undef,
  $has_subrole_db           = true,
  $has_subrole_messaging    = true,
  $has_subrole_api_keystone = true,
  $use_ssl                  = true,
  $rabbit_env               = {},
){

  if $has_subrole_db {

#    package { 'mariadb-clusterchk':
#      ensure => latest,
#      name   => 'mariadb-clusterchk',
#    }

    class { 'galera':
      galera_servers      => $all_masters_ip,
      galera_master       => $first_master,
      package_ensure      => 'latest',
      mysql_package_name  => 'mariadb-server',
      client_package_name => 'default-mysql-client',
      vendor_type         => 'mariadb',
      root_password       => 'a_big_secret',
      status_password     => 'a_big_secret',
      configure_repo      => false,
      configure_firewall  => false,
      galera_package_name => 'galera-3',
      override_options => {
        'mysqld' => {
          'bind_address'                    => '0.0.0.0',
          'wait_timeout'                    => '28800',
          'interactive_timeout'             => '30',
          'connect_timeout'                 => '30',
          'character_set_server'            => 'utf8',
          'collation_server'                => 'utf8_general_ci',
#          'open_files_limit'                => '100000',
#          'max_connections'                 => '400',
#          'max_user_connections'            => '200',
#          'max_connect_errors'              => '999999999',
#          'table_open_cache'                => '4096',
#          'max_allowed_packet'              => '64M',
#          'binlog_cache_size'               => '1M',
#          'max_heap_table_size'             => '32M',
#          'thread_cache_size'               => '32',
#          'query_cache_size'                => '0',
#          'query_cache_type'                => '0',
#          'query_cache_limit'               => '1M',
#          'ft_min_word_len'                 => '3',
#          'transaction_isolation'           => 'REPEATABLE-READ',
#          'tmp_table_size'                  => '32M',
#          'log-bin'                         => 'mysql-bin',
#          'log_bin_trust_function_creators' => '1',
#          'binlog_format'                   => 'ROW',
#          'performance_schema'              => '1',
#          'log_warnings'                    => '2',
#          'max_sp_recursion_depth'          => '100',
#          ## innodb tunning ok for a dedicated int-myd with 64 GB of ram
#          'innodb_buffer_pool_size'         => '40G',
#          'innodb_log_file_size'            => '2G',
#          'innodb_stats_sample_pages'       => '32',
#          'innodb_lock_wait_timeout'        => '10',
#          'innodb_io_capacity'              => '2000',
#          'log_slave_updates'               => '1',
#          # Galera: this should be managed by the galera class already anyway,
#          # so we keep everything commented out.
#          'wsrep_on'                        => 'ON',
#          'wsrep_sst_auth'                  => 'backup:t8rf932fg23',
#          'wsrep_provider'                  => '/usr/lib/galera/libgalera_smm.so'
#          'wsrep_cluster_name'              => 'ocicluster',
#          'innodb_autoinc_lock_mode'        => '2',
#          'innodb_doublewrite'              => '1',
#          # http://galeracluster.com/documentation-webpages/configuration.html
#          'innodb_flush_log_at_trx_commit'  => '0',
#          'expire_logs_days'                => '1',
#          'wsrep_provider_options'          => '"gcache.size = 30G"',
#          'wsrep_sst_donor'                 => 'zabbix-dbnode-2-d2,zabbix-dbnode-1-d3',
#          'wsrep_slave_threads'             => '16',
#          # required for mysql partitioning
#          'event_scheduler': 'OFF',
        }
      }
    }

    class { '::keystone::db::mysql':
      dbname   => 'keystonedb',
      password => 'keystonedbpass',
# This should be set to the hostname of the VIP:
#      host     => ,
    }
  }

#  # RabbitMQ
#  if $has_subrole_messaging {
#    if $use_ssl {
#      class { 'oci::sslkeypair':
#        service_name => 'rabbitmq',
#      }
#      $ssl_cert = "/etc/rabbitmq/ssl/public/${::fqdn}.crt"
#      $ssl_key  = "/etc/rabbitmq/ssl/private/${::fqdn}.key"
#    } else {
#      $ssl_cert = 'UNSET'
#      $ssl_key  = 'UNSET'
#    }
#
#    class { '::rabbitmq':
#      package_ensure           => 'latest',
#      delete_guest_user        => true,
#      ssl                      => $use_ssl,
#      ssl_only                 => $use_ssl,
##     ssl_cacert               => '/etc/bla.ca',
#      ssl_cert                 => $ssl_cert,
#      ssl_key                  => $ssl_key,
#      environment_variables    => $rabbit_env,
#      repos_ensure             => false,
#      # Clustering options...
#      config_cluster           => true,
#      cluster_nodes            => $all_masters,
#      cluster_node_type        => 'ram',
#      erlang_cookie            => 'XAFSRAFNNFJOFSHLEUFT',
#      wipe_db_on_cookie_change => true,
#    }
#
#    rabbitmq_vhost { '/':
#      provider => 'rabbitmqctl',
#      require  => Class['::rabbitmq'],
#    }
#  }
#
#  # Get memcached
#  class { '::memcached':
#    listen_ip => $::fqdn,
#    udp_port  => 0,
#  }
#
#  # Configure Apache to use mod-wsgi-py3, not mod-wsgi
#  if $has_subrole_api_keystone {
#    # Set python3 for mod-wsgi
#    include ::apache::params
#    class { '::apache':
#      mod_packages => merge($::apache::params::mod_packages, {
#        'wsgi' => 'libapache2-mod-wsgi-py3',
#      })
#    }
#  }
#
#  if $has_subrole_api_keystone {
#    class { '::keystone':
#      debug                       => true,
#      database_connection         => 'mysql://keystonedb:keystonedbpass@127.0.0.1/keystone',
#      catalog_type                => 'sql',
#      admin_token                 => 'admin_token',
#      enabled                     => false,
#      enable_ssl                  => $use_ssl,
#      public_bind_host            => "${::fqdn}",
#      admin_bind_host             => "${::fqdn}",
#      rabbit_use_ssl              => $use_ssl,
#      rabbit_ha_queues            => true,
#    }
#  }
}
