CREATE DATABASE IF NOT EXISTS `oci`;

USE `oci`;
CREATE TABLE `blockdevices` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `machine_id` int(11) NOT NULL,
  `name` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  `size_mb` int(11) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=25348 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE `clusters` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  `domain` varchar(253) COLLATE utf8_unicode_ci NOT NULL,
  `first_master_machine_id` int(11) NULL default NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `name` (`name`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE `ifnames` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `machine_id` int(11) NOT NULL,
  `name` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  `macaddr` varchar(20) COLLATE utf8_unicode_ci NOT NULL,
  `max_speed` int(11) NOT NULL default 10,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=25754 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;


CREATE TABLE `ips` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `network` int(11) NOT NULL,
  `ip` bigint(128) NOT NULL,
  `type` enum('4','6') NOT NULL DEFAULT '4',
  `machine` INT(11) NOT NULL,
  `usefor` ENUM('machine','vip') NOT NULL DEFAULT 'machine',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniqueip` (`ip`),
  UNIQUE KEY `uniquemachine` (`network`,`machine`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

CREATE TABLE `machines` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `memory` int(11) NOT NULL,
  `ipaddr` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  `serial` varchar(128) COLLATE utf8_unicode_ci NOT NULL,
  `hostname` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  `installed` enum('yes','no') COLLATE utf8_unicode_ci NOT NULL DEFAULT 'no',
  `lastseen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `status` varchar(128) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'None',
  `role` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  `cluster` int(11) DEFAULT NULL,
  `ipmi_use` enum('yes','no') COLLATE utf8_unicode_ci NOT NULL DEFAULT 'no',
  `ipmi_addr` varchar(254) COLLATE utf8_unicode_ci NOT NULL,
  `ipmi_port` int(11) NOT NULL DEFAULT '623',
  `ipmi_username` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  `ipmi_password` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `serial` (`serial`)
) ENGINE=InnoDB AUTO_INCREMENT=11 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE `networks` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(128) NOT NULL,
  `ip` varchar(64) NOT NULL,
  `cidr` int(3) NOT NULL,
  `is_public` enum('yes','no') NOT NULL DEFAULT 'no',
  `cluster` int(11) DEFAULT NULL,
  `role` varchar(64) DEFAULT NULL,
  `iface1` VARCHAR(32) NULL DEFAULT NULL,
  `iface2` VARCHAR(32) NULL DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `name` (`name`)
) ENGINE=InnoDB AUTO_INCREMENT=6 DEFAULT CHARSET=utf8mb4;

CREATE TABLE `rolecounts` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `cluster` int(11) NOT NULL,
  `role` int(11) NOT NULL,
  `count` int(11) NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `cluster` (`cluster`,`role`)
) ENGINE=InnoDB AUTO_INCREMENT=9 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE `roles` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `name` (`name`)
) ENGINE=InnoDB AUTO_INCREMENT=11 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
INSERT INTO `roles` (`id`, `name`) VALUES (1,'compute');
INSERT INTO `roles` (`id`, `name`) VALUES (2,'controller');
INSERT INTO `roles` (`id`, `name`) VALUES (3,'network');
INSERT INTO `roles` (`id`, `name`) VALUES (4,'volume');
INSERT INTO `roles` (`id`, `name`) VALUES (5,'custom');
