<?php

require_once("read_conf.php");
require_once("db.php");
require_once("ssh.php");
require_once("validate_param.php");
require_once("slave_actions.php");

$conf = readmyconf();
$con = connectme($conf);

function seconds_to_duration ($seconds){
    if(($seconds == 0) || ($seconds == 1)){
        return "$seconds sec";
    }
    if($seconds < 120){
        return "$seconds secs";
    }
    $minutes = floor($seconds / 60);
    if($minutes < 120){
        return "$minutes mins";
    }
    $hours = floor($minutes/60);
    if($hours < 48){
        return "$hours hours";
    }
    $days = floor($hours / 24);
    return "$days days";
}

function mb_to_smart_size ($mb) {
    if($mb < 4096){
        return $mb. " MB";
    }
    $gb = floor($mb / 1024);
    if($gb < 4096){
        return $gb . " GB";
    }
    $tb = floor($gb / 1024);
    return $tb . " TB";
}

function machine_list($con){
    $q = "SELECT * FROM clusters ORDER BY id";
    $r = mysqli_query($con, $q);
    $n = mysqli_num_rows($r);
    $clusters = [];
    for($i=0; $i<$n; $i++){
        $a = mysqli_fetch_array($r);
        $clusters[ $a["id"] ] = $a;
    }

    if(isset($_REQUEST["subaction"]) && $_REQUEST["subaction"] == "edit_ipmi"){
        $mlist = "<h2>Edit IPMI settings for this machine</h2>";
        $safe_machine_id = safe_int("machine_id");
        if($safe_machine_id === FALSE){
            $mlist .= "Machine ID in wrong format.";
            return $mlist;
        }
        $q = "SELECT * FROM machines WHERE id='$safe_machine_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $mlist .= "Cannot find machine by this ID.";
            return $mlist;
        }
        $a = mysqli_fetch_array($r);

        if($a["cluster"] != NULL){
            $clid    = $a["cluster"];
            $cluster = $clusters[$clid]["name"];
        }else{
            $cluster = "Not in a cluster";
        }

        $ipaddr   = $a["ipaddr"];
        $serial   = $a["serial"];
        $role     = $a["role"];
        $hostname = $a["hostname"];
        $ipmi_use      = $a["ipmi_use"];
        $ipmi_addr     = $a["ipmi_addr"];
        $ipmi_port     = $a["ipmi_port"];
        $ipmi_username = $a["ipmi_username"];
        $ipmi_password = $a["ipmi_password"];

        if($ipmi_use == "yes"){
            $selector = " checked";
        }else{
            $selector = "";
        }

        $mlist .= "<table><tr><th>IP address:</th><td>$ipaddr</td></tr>";
        $mlist .= "<th>Serial:</th><td>$serial</td></tr>";
        $mlist .= "<th>Hostname:</th><td>$hostname</td></tr>";
        $mlist .= "<th>Cluster:</th><td>$cluster</td></tr>";
        $mlist .= "<form action='?'><input type='hidden' name='action' value='set_ipmi'><input type='hidden' name='machine_id' value='$safe_machine_id'>";
        $mlist .= "<th>Use IPMI:</th><td><input type='checkbox' name='ipmi_use' value='yes'$selector></td></tr>";
        $mlist .= "<th>IPMI address:</th><td><input type='text' name='ipmi_addr' value='$ipmi_addr'></td></tr>";
        $mlist .= "<th>IPMI port:</th><td><input type='text' name='ipmi_port' value='$ipmi_port'></td></tr>";
        $mlist .= "<th>IPMI username:</th><td><input type='text' name='ipmi_username' value='$ipmi_username'></td></tr>";
        $mlist .= "<th>IPMI password:</th><td><input type='text' name='ipmi_password' value='$ipmi_password'></td></tr>";
        $mlist .= "<th></th><td><input type='submit' name='save' value='Save'></td></tr>";
        $mlist .= "</form></table>";
        return $mlist;
    }

    $mlist = "<table border='1'>
<tr><th>IP address<br>Serial</th><th>IPMI</th><th>Memory</th><th>HDD</th><th>iface</th><th>status</th><th>Last seen</th><th>Role</th><th>Cluster</th><th>Hostname</th><th>Action</th><th>Reboots</th></tr>\n";
    $r = mysqli_query($con, "SELECT id,memory,ipaddr,serial,hostname,installed,lastseen,status,role,cluster, TIME_TO_SEC(TIMEDIFF(NOW(),lastseen)) AS timediff,ipmi_use FROM machines ORDER BY serial");
    $n = mysqli_num_rows($r);
    for($i=0; $i<$n; $i++){
        $a = mysqli_fetch_array($r);
        $machine_id = $a["id"];
        $ipaddr     = $a["ipaddr"];
        $serial     = $a["serial"];
        $role       = $a["role"];
        $ipmi_use   = $a["ipmi_use"];
        if($a["cluster"] != NULL){
            $clid    = $a["cluster"];
            $cluster = $clusters[$clid]["name"];
        }else{
            $cluster = "Not in a cluster";
        }

        $mlist .= "<tr>";
        $mlist .= "<td>$ipaddr<br>$serial</td>";
        $mlist .= "<td><a href='?subaction=edit_ipmi&machine_id=$machine_id'>". $a["ipmi_use"] ."</a></td>";
        $mlist .= "<td>". mb_to_smart_size($a["memory"]) ."</td>";

        $blockdevs = "";
        $r2 = mysqli_query($con, "SELECT * FROM blockdevices WHERE machine_id='".$a["id"]."'");
        $n2 = mysqli_num_rows($r2);
        for($i2=0; $i2<$n2; $i2++){
            $a2 = mysqli_fetch_array($r2);
            $blockdevs .= $a2["name"] . ": " . mb_to_smart_size($a2["size_mb"]) . "<br>";
        }

        $mlist .= "<td>".$blockdevs."</td>";

        $ifaces = "";
        $r2 = mysqli_query($con, "SELECT * FROM ifnames WHERE machine_id='".$a["id"]."'");
        $n2 = mysqli_num_rows($r2);
        for($i2=0; $i2<$n2; $i2++){
            $a2 = mysqli_fetch_array($r2);
            if($a2["max_speed"] >= 1000){
              $speed = $a2["max_speed"] / 1000 . "Gb/s";
            }else{
              $speed = $a2["max_speed"] . "Mb/s";
            }
            $ifaces .= $a2["name"] . ": " . $a2["macaddr"] . ", " . $speed . "<br>";
        }

        $mlist .= "<td>$ifaces</td>";
        $mlist .= "<td>".$a["status"]."</td>";
        $mlist .= "<td>".seconds_to_duration($a["timediff"])."</td>";
        $mlist .= "<td>$role</td>";
        $mlist .= "<td>$cluster</td>";
        # HOSTNAME field or value
        if($a["status"] == "live"){
            $mlist .= "<td><form action='?'><input type='text' size='40' name='hostname' value='".$a["hostname"]."'></td>";
        }else{
            $mlist .= "<td>".$a["hostname"]."</td>";
        }
        if($ipmi_use == "yes"){
            $ipmi_powercycle_button = "<form action='?'><td><input type='hidden' name='action' value='ipmi_reboot_on_hdd'><input type='hidden' name='ipaddr' value='$ipaddr'><input type='submit' value='IPMI reboot on HDD'></td></form>";
            $ipmi_powercycle_button .= "<form action='?'><td><input type='hidden' name='action' value='ipmi_reboot_on_live'><input type='hidden' name='ipaddr' value='$ipaddr'><input type='submit' value='IPMI reboot on live'></td></form>";
        }else{
            $ipmi_powercycle_button = "";
        }
        # INSTALL button
        switch($a["status"]){
        case "live":
            $mlist .= "<td><input type='hidden' name='action' value='install_os'><input type='hidden' name='ipaddr' value='$ipaddr'><input type='submit' text='Install' value='Install'></form></td>";
            break;
        case "installing":
            $mlist .= "<td><a target='_blank' href='view_log.php?ipaddr=$ipaddr'>view log</a></td>";
            break;
        default:
            $mlist .= "<td></td>";
            break;
        }
        # REBOOT button
        switch($a["status"]){
        case "live":
            $mlist .= "<td><center><table><tr><form action='?'><td><input type='hidden' name='action' value='reboot_on_hdd'><input type='hidden' name='ipaddr' value='$ipaddr'><input type='submit' text='Reboot on HDD' value='Reboot on HDD'></td></form>$ipmi_powercycle_button</tr></table></center></td>";
            break;
        case "installed":
            $mlist .= "<td><center><table><tr><form action='?'><td><input type='hidden' name='action' value='reboot_on_live'><input type='hidden' name='ipaddr' value='$ipaddr'><input type='submit' text='Reboot on live' value='Reboot on Live'></td></form>$ipmi_powercycle_button</tr></table></center></td>";
            break;
        case "installing":
        case "firstboot":
        default:
            $mlist .= "<td><table>$ipmi_powercycle_button</table></td>";
            break;
        }
        $mlist .= "</tr>\n";
    }
    $mlist .= "</table>\n";
    return $mlist;
}

function cluster_list($con,$conf){
    $clist = "<table border='1'>
<tr><th>Name</th><th>Domain</th><th>Action</th></tr>\n";
    $r = mysqli_query($con, "SELECT * FROM clusters");
    $n = mysqli_num_rows($r);
    for($i=0; $i<$n; $i++){
        $a = mysqli_fetch_array($r);
        $clist .= "<tr><td>".$a["name"]."</td><td>".$a["domain"]."</td><td><form action='?'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='delete_cluster'><input type='hidden' name='id' value='".$a["id"]."'><input type='submit' text='Delete cluster' value='Delete cluster'></form></td></tr>\n";
    }
    $clist .= "<tr><form action='?'><td><input type='hidden' name='h-open-tab' value='Clusters'><input type='text' size='40' name='name' value=''></td><td><input type='text' size='40' name='domain' value=''></td>";
    $clist .= "<td><center><input type='hidden' name='action' value='new_cluster'><input type='submit' text='New cluster' value='New cluster'></form></center></td></tr>\n";
    $clist .= "</table>\n";
    return $clist;
}

function get_machine_id_from_ip($con, $safe_ip){
    $r = mysqli_query($con, "SELECT * FROM machines WHERE ipaddr='$safe_ip'");
    $n = mysqli_num_rows($r);
    $machine = mysqli_fetch_array($r);
    return $machine["id"];
}

function perform_actions($con,$conf){
    $out = "";
    if(!isset($_REQUEST["action"])){
        return $out;
    }

    switch($_REQUEST["action"]){
    case "install_os":
        # Validate IPv4
        $safe_ip = safe_ipv4("ipaddr");
        if($safe_ip === FALSE){
            $out .= "Wrong ipv4 format.";
            return $out;
        }
        if(check_machine_with_ip_exists($con, $safe_ip) === FALSE){
            $out .= "Cannot find machine with this IP address.";
            return $out;
        }
        # Get ID from IPv4
        $machine_id = get_machine_id_from_ip($con, $safe_ip);

        # Validate hostname, commit it to DB before calculating
        # command line and launching install
        $safe_fqdn = safe_fqdn("hostname");
        if($safe_fqdn === FALSE){
            $out = "Missing or wrong hostname.";
            return $out;
        }
        $r = mysqli_query($con, "UPDATE machines SET hostname='$safe_fqdn' WHERE ipaddr='$safe_ip'");

        $slave_install_return = slave_install_server_os_command($con, $conf, $machine_id);
        if($slave_install_return["status"] != "success"){
          return "Error while calculating installation command line for host $safe_fqdn: ".$slave_install_return["message"];
        }
        $out .= "Running: ". $slave_install_return["cmd"];
        slave_install_os($con, $conf, $machine_id, $slave_install_return["cmd"]);
        return $out;
        break;
    case "reboot_on_live":
        # Validate IPv4
        $safe_ip = safe_ipv4("ipaddr");
        if($safe_ip === FALSE){
            $out .= "Wrong ipv4 format.";
            return $out;
        }
        if(check_machine_with_ip_exists($con, $safe_ip) === FALSE){
            $out .= "Cannot find machine with this IP address.";
            return $out;
        }
        $r = mysqli_query($con, "UPDATE machines SET status='bootinglive' WHERE ipaddr='$safe_ip'");
        $ret = send_ssh_cmd($conf, $con, $_REQUEST["ipaddr"], "shutdown -r now");
        break;
    case "reboot_on_hdd":
        # Validate IPv4
        $safe_ip = safe_ipv4("ipaddr");
        if($safe_ip === FALSE){
            $out .= "Wrong ipv4 format.";
            return $out;
        }
        if(check_machine_with_ip_exists($con, $safe_ip) === FALSE){
            $out .= "Cannot find machine with this IP address.";
            return $out;
        }
        $r = mysqli_query($con, "UPDATE machines SET status='firstboot' WHERE ipaddr='$safe_ip'");
        $ret = send_ssh_cmd($conf, $con, $_REQUEST["ipaddr"], "shutdown -r now");
        break;
    case "ipmi_reboot_on_hdd":
        // ipmitool -I lanplus -H 192.168.100.1 -U ipmiusr -p 9002 -P test power cycle
        $safe_ip = safe_ipv4("ipaddr");
        if($safe_ip === FALSE){
            $out .= "Wrong ipv4 format.";
            return $out;
        }
        if(check_machine_with_ip_exists($con, $safe_ip) === FALSE){
            $out .= "Cannot find machine with this IP address.";
            return $out;
        }
        $r = mysqli_query($con, "UPDATE machines SET status='firstboot' WHERE ipaddr='$safe_ip'");
        $q = "SELECT * FROM machines WHERE ipaddr='$safe_ip'";
        $r = mysqli_query($con, $q);
        $machine = mysqli_fetch_array($r);
        $ipmi_addr     = $machine["ipmi_addr"];
        $ipmi_port     = $machine["ipmi_port"];
        $ipmi_username = $machine["ipmi_username"];
        $ipmi_password = $machine["ipmi_password"];
        $cmd = "ipmitool -I lanplus -H $ipmi_addr -p $ipmi_port -U $ipmi_username -P $ipmi_password power cycle";
        $output = "";
        $return_var = 0;
        exec($cmd, $output, $return_var);
        break;
    case "ipmi_reboot_on_live":
        // ipmitool -I lanplus -H 192.168.100.1 -U ipmiusr -p 9002 -P test power cycle
        $safe_ip = safe_ipv4("ipaddr");
        if($safe_ip === FALSE){
            $out .= "Wrong ipv4 format.";
            return $out;
        }
        if(check_machine_with_ip_exists($con, $safe_ip) === FALSE){
            $out .= "Cannot find machine with this IP address.";
            return $out;
        }
        $r = mysqli_query($con, "UPDATE machines SET status='bootinglive' WHERE ipaddr='$safe_ip'");
        $q = "SELECT * FROM machines WHERE ipaddr='$safe_ip'";
        $r = mysqli_query($con, $q);
        $machine = mysqli_fetch_array($r);
        $ipmi_addr     = $machine["ipmi_addr"];
        $ipmi_port     = $machine["ipmi_port"];
        $ipmi_username = $machine["ipmi_username"];
        $ipmi_password = $machine["ipmi_password"];
        $cmd = "ipmitool -I lanplus -H $ipmi_addr -p $ipmi_port -U $ipmi_username -P $ipmi_password power cycle";
        $output = "";
        $return_var = 0;
        exec($cmd, $output, $return_var);
        break;
    case "new_cluster":
        $safe_domain = safe_fqdn("domain");
        if($safe_domain === FALSE){
            $out .= "Domain name in wrong format.";
            return $out;
        }
        $safe_cluster = safe_cluster_name("name");
        if($safe_cluster === FALSE){
            $out .= "Cluster name in wrong format.";
        }
        $q = "INSERT INTO clusters (id, name, domain) VALUES ('', '$safe_cluster', '$safe_domain'); ";
        $r = mysqli_query($con, $q);
        break;
    case "delete_cluster":
        $safe_id = safe_int("id");
        if($safe_id === FALSE){
            $out .= "Cluster ID in wrong format.";
            return $out;
        }
        if(check_cluster_with_id_exists($con, $safe_id) === FALSE){
            $out .= "Cluster with ID $safe_id not found.";
            return $out;
        }
        $q = "DELETE FROM clusters WHERE id='$safe_id'";
        $r = mysqli_query($con, $q);
        $q = "UPDATE machines SET cluster=NULL, role='' WHERE cluster='$safe_id'";
        $r = mysqli_query($con, $q);
        $q = "DELETE FROM rolecounts WHERE cluster='$safe_id'";
        $r = mysqli_query($con, $q);
        break;
    case "remove_from_cluster":
        // TODO: Check validity of cluster-id and machine-id in the DB before the UPDATE call
        $safe_machine_id = safe_int("machine-id");
        if($safe_machine_id === FALSE){
            $out .= "Machine ID in wrong format.";
            return $out;
        }
        $q = "SELECT role, cluster FROM machines WHERE id='$safe_machine_id';";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n == 0){
            $out = "No machine by this ID.";
            return $out;
        }
        $machine = mysqli_fetch_array($r);
        $safe_role = $machine["role"];
        $safe_cluster_id = $machine["cluster"];

        $q = "UPDATE machines SET cluster=NULL WHERE id='$safe_machine_id'";
        $r = mysqli_query($con, $q);

        // If role is controller, check if there's already controller. If there's none, then
        // we must set a controller as first master
        if($safe_role == "controller"){
            $q = "SELECT first_master_machine_id FROM clusters WHERE id='$safe_cluster_id'";
            $r = mysqli_query($con, $q);
            $cluster = mysqli_fetch_array($r);
            $fm_id = $cluster["first_master_machine_id"];;
            if($fm_id == $safe_machine_id){
                $q = "SELECT * FROM machines WHERE cluster='$safe_cluster_id' AND role='controller' LIMIT 1;";
                $r = mysqli_query($con, $q);
                $n = mysqli_num_rows($r);
                // No controler left: we set the first_master as NULL
                if($n == 0){
                    $q = "UPDATE clusters SET first_master_machine_id=NULL WHERE id='$safe_cluster_id'";
                    $r = mysqli_query($con, $q);
                // We just set the first controler we find left as first_master
                }else{
                    $first_master = mysqli_fetch_array($r);
                    $new_fm_id = $first_master["id"];
                    $q = "UPDATE clusters SET first_master_machine_id='$new_fm_id' WHERE id='$safe_cluster_id'";
                    $r = mysqli_query($con, $q);
                }
            }
        }
        // Delete all IPs of the machine in the DB
        $q = "DELETE FROM ips WHERE machine='$safe_machine_id'";
        $r = mysqli_query($con, $q);
        return $out;
        break;
    case "add_machines_to_cluster":
        // Request like: Array ( [id] => Array ( [0] => 5 [1] => 6 ) [role] => controller [cluster-id] => 2 ) 
        // Validate cluster ID, and fetch its name and domain
        $safe_cluster_id = safe_int("cluster-id");
        if($safe_cluster_id === FALSE){
            $out .= "Cluster ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM clusters WHERE id='$safe_cluster_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cluster with id $safe_cluster_id not found.";
            return $out;
        }
        $cluster = mysqli_fetch_array($r);
        $cluster_name = $cluster["name"];
        $cluster_domain = $cluster["domain"];

        // Validate roles
        if(!isset($_REQUEST["role"])){
            $out .= "No role selected.";
            return $out;
        }
        $safe_role = safe_fqdn("role");
        if($safe_role === FALSE){
            $out .= "Role not in correct format.";
            return $out;
        }
        $q = "SELECT * FROM roles WHERE name='$safe_role'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Role not found in db.";
            return $out;
        }

        // Find role count
        $a = mysqli_fetch_array($r);
        $safe_role_id = $a["id"];
        $q = "SELECT * FROM rolecounts WHERE cluster='$safe_cluster_id' AND role='$safe_role_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n == 0){
            $role_count = 0;
            $q = "INSERT INTO rolecounts (id, cluster, role, count) VALUES ('', '$safe_cluster_id', '$safe_role_id', '0')";
            $r = mysqli_query($con, $q);
            $role_count_id = mysqli_insert_id($con);
        }else{
            $a = mysqli_fetch_array($r);
            $role_count = $a["count"];
            $role_count_id = $a["id"];
        }

        // Validate machine IDs.
        $num_machines = 0;
        foreach($_REQUEST['id'] as $machine_id){
            $reg = '/^[0-9]{1,12}$/';
	    if(!preg_match($reg,$machine_id)){
	        $out .= "Machine IDs in wrong format.";
	        return $out;
            }
            $num_machines += 1;
        }
        $safe_machine_id_array = $_REQUEST['id'];

        // If role is controller, check if there's already controller. If there's none, then
        // we must set a controller as first master
        if($safe_role == "controller"){
            $q_fm = "SELECT id FROM machines WHERE role='controller' AND cluster='$safe_cluster_id'";
            $r_fm = mysqli_query($con, $q_fm);
            $n_fm = mysqli_num_rows($r_fm);
        }

        // For all machines, set role, cluster and hostname
        foreach($safe_machine_id_array as $safe_machine_id){
            $role_count += 1;
            $hostname = $cluster_name . "-" . $safe_role . "-node-" . $role_count . "." . $cluster_domain;
            $q = "UPDATE machines SET role='$safe_role', cluster='$safe_cluster_id', hostname='$hostname' WHERE id='$safe_machine_id'";
            $r = mysqli_query($con, $q);
            if($safe_role == "controller" && $n_fm == 0){
                $q = "UPDATE clusters SET first_master_machine_id='$safe_machine_id' WHERE id='$safe_cluster_id'";
                $r = mysqli_query($con, $q);
                $n_fm = 1;
            }
            // Assign IPs to it
            $ret = slave_assign_all_networks_ip_addresses($con, $conf, $safe_machine_id, "machine");
            if($ret["status"] != "success"){
                echo "<font color='white'>Here: ". $ret["message"] ."</font>";
                return $ret["message"];
            }
        }
        $q = "UPDATE rolecounts SET count='$role_count' WHERE id='$role_count_id'";
        $r = mysqli_query($con, $q);
        return $out;
        break;
    case "add_role":
        $safe_role_name = safe_fqdn("name");
        if($safe_role_name === FALSE){
            $out .= "Role name in wrong format.";
            return $out;
        }
        $q = "INSERT INTO roles (name) VALUES ('$safe_role_name')";
        $r = mysqli_query($con, $q);
        break;
    case "delete_role":
        $safe_role_id = safe_int("id");
        if($safe_role_id === FALSE){
            $out .= "Role id is wrong.";
            return $out;
        }
        $q = "DELETE FROM roles WHERE id='$safe_role_id'";
        $r = mysqli_query($con, $q);
        break;
    case "select_frist_master_id":
        $safe_fm_id = safe_int("first_master_id");
        if($safe_fm_id === FALSE){
            $out .= "Wrong format for first master ID.";
            return $out;
        }
        $safe_cluster_id = safe_int("cluster-id");
        if($safe_cluster_id === FALSE){
            $out .= "Wrong format for cluster ID.";
            return $out;
        }
        $q = "SELECT * FROM machines WHERE cluster='$safe_cluster_id' AND id='$safe_fm_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n == 0){
            $out .= "No such machine in the cluster.";
            return $out;
        }
        $q = "UPDATE clusters SET first_master_machine_id='$safe_fm_id' WHERE id='$safe_cluster_id'";
        $r = mysqli_query($con, $q);
        break;
    case "create_network":
        $safe_network_name = safe_fqdn("name");
        if($safe_network_name === FALSE){
            $out .= "Network name in wrong format.";
            return $out;
        }
        $safe_ip = safe_ipv4("ip");
        if($safe_ip === FALSE){
            $out .= "IP in wrong format.";
            return $out;
        }
        $safe_cidr = safe_int("cidr");
        if($safe_cidr === FALSE || $safe_cidr < 8 || $safe_cidr > 32){
            $out .= "CIDR in wrong format.";
            return $out;
        }
        if( isset($_REQUEST["is_public"]) && $_REQUEST["is_public"] == "yes"){
            $safe_is_public = "yes";
        }else{
            $safe_is_public = "no";
        }
        $q = "INSERT INTO networks (id, name, ip, cidr, is_public) VALUES ('', '$safe_network_name', '$safe_ip', '$safe_cidr', '$safe_is_public')";
        $r = mysqli_query($con, $q);
        break;
    case "edit_network":
        $safe_network_name = safe_fqdn("name");
        if($safe_network_name === FALSE){
            $out .= "Network name in wrong format.";
            return $out;
        }
        $safe_ip = safe_ipv4("ip");
        if($safe_ip === FALSE){
            $out .= "IP in wrong format.";
            return $out;
        }
        $safe_cidr = safe_int("cidr");
        if($safe_cidr === FALSE || $safe_cidr < 8 || $safe_cidr > 32){
            $out .= "CIDR in wrong format.";
            return $out;
        }
        $safe_id = safe_int("id");
        if($safe_id === FALSE){
            $out .= "Network ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM networks WHERE id='$safe_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find network by that ID.";
            return $out;
        }
        if( isset($_REQUEST["is_public"]) && $_REQUEST["is_public"] == "yes"){
            $safe_is_public = "yes";
        }else{
            $safe_is_public = "no";
        }
        $q = "UPDATE networks SET name='$safe_network_name', ip='$safe_ip', cidr='$safe_cidr', is_public='$safe_is_public' WHERE id='$safe_id';";
        $r = mysqli_query($con, $q);
        break;
    case "delete_network":
        $safe_id = safe_int("id");
        if($safe_id === FALSE){
            $out .= "ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM networks WHERE id='$safe_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find network by that ID.";
            return $out;
        }
        $q = "DELETE FROM networks WHERE id='$safe_id';";
        $r = mysqli_query($con, $q);
        break;
    case "add_network_to_cluster":
        $safe_network_id = safe_int("network_id");
        if($safe_network_id === FALSE){
            $out .= "Network ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM networks WHERE id='$safe_network_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find network by that ID.";
            return $out;
        }
        $safe_role_name = safe_fqdn("role");
        if($safe_role_name === FALSE){
            $out .= "Role name in wrong format.";
            return $out;
        }
        if($safe_role_name == "all"){
            $sql_role = "'all'";
        }else{
            $q = "SELECT * FROM roles WHERE name='$safe_role_name'";
            $r = mysqli_query($con, $q);
            $n = mysqli_num_rows($r);
            if($n != 1){
                $out .= "Cannot find role by that name.";
                return $out;
            }
            $sql_role = "'$safe_role_name'";
        }

        $safe_cluster_id = safe_int("cluster_id");
        if($safe_cluster_id === FALSE){
            $out .= "Cluster ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM clusters WHERE id='$safe_cluster_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find cluster by that ID.";
            return $out;
        }

        $safe_iface1 = safe_fqdn("iface1");
        if($safe_cluster_id === FALSE){
            $out .= "Iface 1 in wrong format.";
            return $out;
        }
        $safe_iface2 = safe_fqdn("iface2");
        if($safe_cluster_id === FALSE){
            $out .= "Iface 1 in wrong format.";
            return $out;
        }

        $q = "UPDATE networks SET cluster='$safe_cluster_id', role=$sql_role, iface1='$safe_iface1', iface2='$safe_iface2' WHERE id='$safe_network_id'";
        $r = mysqli_query($con, $q);
        // Reserve the VIP
        if($safe_role_name == "all" || $safe_role_name == "controller"){
            $ret = reserve_ip_address($con, $conf, $safe_network_id, 0, "vip");
            if($ret["status"] != "success"){
                return $ret["message"];
            }
        }
        $ret = reserve_ip_to_all_slaves_of_network($con, $conf, $safe_cluster_id, $safe_network_id, $safe_role_name);
        if($ret["status"] != "success"){
            return $ret["message"];
        }
        break;
    case "delete_network_from_cluster":
        // http://bdbdev-ogiz.infomaniak.ch/oci/?v-open-tab=zigo&h-open-tab=Clusters&action=delete_network_from_cluster&cluster_id=7&network_id=4
        $safe_network_id = safe_int("network_id");
        if($safe_network_id === FALSE){
            $out .= "Network ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM networks WHERE id='$safe_network_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find network by that ID.";
            return $out;
        }
        
        $safe_cluster_id = safe_int("cluster_id");
        if($safe_cluster_id === FALSE){
            $out .= "Cluster ID in wrong format.";
            return $out;
        }
        $q = "SELECT * FROM clusters WHERE id='$safe_cluster_id'";
        $r = mysqli_query($con, $q);
        $n = mysqli_num_rows($r);
        if($n != 1){
            $out .= "Cannot find cluster by that ID.";
            return $out;
        }

        $q = "UPDATE networks SET cluster=NULL, role=NULL WHERE id='$safe_network_id'";
        $r = mysqli_query($con, $q);
        $q = "DELETE FROM ips WHERE network='$safe_network_id'";
        $r = mysqli_query($con, $q);
        break;
    case "set_ipmi":
        // http://bdbdev-ogiz.infomaniak.ch/oci/?action=set_ipmi&machine_id=18&ipmi_use=yes&ipmi_addr=192.168.100.1&ipmi_port=623&ipmi_username=ipmiusr&ipmi_password=test&save=Save
        $safe_machine_id = safe_int("machine_id");
        if($safe_machine_id === FALSE){
            $out .= "Machine id is wrong.";
            return $out;
        }
        if(!isset($_REQUEST["ipmi_use"])){
            $q = "UPDATE machines SET ipmi_use='no' WHERE id='$safe_machine_id'";
            $r = mysqli_query($con, $q);
            return $out;
        }
        $safe_ipmi_addr = safe_fqdn("ipmi_addr");
        if($safe_ipmi_addr === FALSE){
            $safe_ipmi_addr = safe_ipv4("ipmi_addr");
            if($safe_ipmi_addr === FALSE){
                $out .= "IPMI address in wrong format.";
                return $out;
            }
        }
        $safe_ipmi_port = safe_int("ipmi_port");
        if($safe_ipmi_port === FALSE){
            $out .= "IPMI port in wrong format.";
            return $out;
        }
        $safe_ipmi_username = safe_fqdn("ipmi_username");
        if($safe_ipmi_username === FALSE){
            $out .= "IPMI username in wrong format.";
            return $out;
        }
        $safe_ipmi_password = safe_password("ipmi_password");
        if($safe_ipmi_password === FALSE){
            $out .= "IPMI password in wrong format.";
            return $out;
        }
        $q = "UPDATE machines SET ipmi_use='yes', ipmi_addr='$safe_ipmi_addr', ipmi_port='$safe_ipmi_port', ipmi_username='$safe_ipmi_username', ipmi_password='$safe_ipmi_password' WHERE id='$safe_machine_id'";
        $r = mysqli_query($con, $q);
    default:
        break;
    }
}

$out_message = perform_actions($con,$conf);

function clusterButtonList($con){
    $out = "";
    $r = mysqli_query($con, "SELECT * FROM clusters");
    $n = mysqli_num_rows($r);
    for($i=0;$i<$n;$i++){
        $a = mysqli_fetch_array($r);
        $name = $a["name"];
        $domain = $a["domain"];
        $out .= "<button id=\"".$name."Bt\" class=\"vtablinks\" onclick=\"openVTab(event, '".$name."')\" >$name&nbsp;$domain</button>\n";
    }
    return $out;
}

function role_selector($con, $conf, $role, $has_all){
    $out = "";
    $r = mysqli_query($con, "SELECT * FROM roles");
    $n = mysqli_num_rows($r);
    $out .= "<select name='role'>";
    if($has_all == "yes"){
        $out .= "<option value='all'>All</option>\n";
        $out .= "<option value='norole'>No role</option>\n";
    }
    for($i=0;$i<$n;$i++){
        $a = mysqli_fetch_array($r);
        $name = $a["name"];
        if($role == $name){
            $selected = " selected";
        }else{
            $selected = "";
        }
        $out .= "<option value='$name'$selected>$name</option>\n";
    }
    $out .= "</select>";
    return $out;
}

function cluster_config_list($con,$conf){
    $out = "";
    $r = mysqli_query($con, "SELECT * FROM clusters");
    $n = mysqli_num_rows($r);
    for($i=0;$i<$n;$i++){
        $a = mysqli_fetch_array($r);
        $name = $a["name"];
        $cluster_id = $a["id"];
        $cluster_fm = $a["first_master_machine_id"]; 
        $domain = $a["domain"];
        $out .= "<div id=\"$name\" class=\"vtabcontent\" style=\"display:none;\">\n";

        // General cluster info
        $out .= "<table border='0'><tr><td valign='top'>";
        
        $out .= "<h2>Machines count</h2>";
        // Get all role counts for this cluster
        $q = "SELECT * FROM rolecounts WHERE cluster='$cluster_id'";
        $r_role_counts = mysqli_query($con, $q);
        $n_roles = mysqli_num_rows($r_role_counts);
        for($j=0;$j<$n_roles;$j++){
            // Fetch matching role name
            $a_role = mysqli_fetch_array($r_role_counts);
            $role_id = $a_role["role"];
            $q = "SELECT name FROM roles WHERE id='$role_id'";
            $r_role_name = mysqli_query($con, $q);
            $a_my_role = mysqli_fetch_array($r_role_name);
            $role_name = $a_my_role["name"];
            // Count how many machines by that role on this cluster
            $q = "SELECT count(role) AS rolecount FROM machines WHERE cluster='$cluster_id' AND role='$role_name'";
            $res = mysqli_query($con, $q);
            $row = mysqli_fetch_row($res);
            $role_count = $row[0];
            if($role_count != 0){
                $out .= $role_name . ": $role_count<br>";
            }
        }

        $out .= "</td><td valign='top'>";

        /////////////////////////////////
        // First master selection form //
        /////////////////////////////////
        $q = "SELECT * FROM machines WHERE cluster='$cluster_id' AND role='controller'";
        $r_fm = mysqli_query($con, $q);
        $n_fm = mysqli_num_rows($r_fm);

        if($n_fm > 0){
            $out .= "<h2>Cluster's first master</h2>";
            $out .= "<form action='?'><input type='hidden' name='v-open-tab' value='$name'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='select_frist_master_id'><input type='hidden' name='cluster-id' value='$cluster_id'><select name='first_master_id'>";
            for($j=0;$j<$n_fm;$j++){
                $a_fm = mysqli_fetch_array($r_fm);
                $fm_id = $a_fm["id"];
                $fm_hostname = $a_fm["hostname"];
                if($cluster_fm == $fm_id){
                    $selector = " selected";
                }else{
                    $selector = "";
                }
                $out .= "<option value='$fm_id'$selector>$fm_hostname</option>";
            }
            $out .= "</select><input type='submit' name='submit' value='Ok'></form>";
        }

        $out .= "</td><td valign='top'>";

        ////////////////////////////
        // Cluster's network form //
        ////////////////////////////
        $out .= "<h2>Cluster's networks</h2>";


        $out .= "<table border='1'>
<tr><th>Network</th><th>VIP</th><th>Role</th><th>Interfaces &amp; bonds</th><th>Action</th></tr>\n";


        $q_n = "SELECT * FROM networks WHERE cluster='$cluster_id'";
        $r_n = mysqli_query($con, $q_n);
        $n_n = mysqli_num_rows($r_n);
        for($j=0;$j<$n_n;$j++){
            $a_n = mysqli_fetch_array($r_n);
            $n_id   = $a_n["id"];
            $n_name = $a_n["name"];
            $n_ip   = $a_n["ip"];
            $n_cidr = $a_n["cidr"];
            $n_role = $a_n["role"];
            $n_iface1 = $a_n["iface1"];
            $n_iface2 = $a_n["iface2"];
            $n_ifaces = $n_iface1;
            if($n_iface2 != "none"){
                $n_ifaces .= " bond with " . $n_iface2;
            }
            if(is_null($n_role)){
                $n_role = "All roles";
            }
            $q_vip = "SELECT INET_NTOA(ip) AS addr FROM ips WHERE network='$n_id' AND usefor='vip'";
            $r_vip = mysqli_query($con, $q_vip);
            $n_vip = mysqli_num_rows($r_vip);
            if($n_vip == 1){
                $a_vip = mysqli_fetch_array($r_vip);
                $vip_addr = $a_vip["addr"];
            }else{
                $vip_addr = "";
            }

            $out .= "<tr><td>$n_name: $n_ip/$n_cidr</td><td>$vip_addr</td><td>$n_role</td><td>$n_ifaces</td><td><form action='?'><input type='hidden' name='v-open-tab' value='$name'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='delete_network_from_cluster'><input type='hidden' name='cluster_id' value='$cluster_id'><input type='hidden' name='network_id' value='$n_id'><input type='submit' value='Delete'></form></td></tr>";
        }

        $q_n = "SELECT * FROM networks WHERE cluster IS NULL";
        $r_n = mysqli_query($con, $q_n);
        $n_n = mysqli_num_rows($r_n);
        if($n_n > 0){
            $out .= "<tr><td>";
            $out .= "<form action='?'><input type='hidden' name='v-open-tab' value='$name'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='add_network_to_cluster'><input type='hidden' name='cluster_id' value='$cluster_id'><select name='network_id'>";
            for($j=0;$j<$n_n;$j++){
                $a_n = mysqli_fetch_array($r_n);
                $n_id   = $a_n["id"];
                $n_name = $a_n["name"];
                $n_ip   = $a_n["ip"];
                $n_cidr = $a_n["cidr"];
                $out .= "<option value='$n_id'>$n_name: $n_ip/$n_cidr</option>";
            }
            $out .= "<select>";
            $out .= "</td><td></td><td>".role_selector($con, $conf, "none", "yes")."</td>";
            $out .= "<td><select name='iface1'>
<option value='eth0'>eth0</option>
<option value='eth1'>eth1</option>
<option value='eth2'>eth2</option>
<option value='eth3'>eth3</option>
<option value='eth4'>eth4</option>
<option value='eth5'>eth5</option>
<option value='10m1'>10 Mb/s, 1st card</option>
<option value='10m2'>10 Mb/s, 2nd card</option>
<option value='10m3'>10 Mb/s, 3rd card</option>
<option value='10m4'>10 Mb/s, 4th card</option>
<option value='100m1'>100 Mb/s, 1st card</option>
<option value='100m2'>100 Mb/s, 2nd card</option>
<option value='100m3'>100 Mb/s, 3rd card</option>
<option value='100m4'>100 Mb/s, 4th card</option>
<option value='1g1'>1 Gb/s, 1st card</option>
<option value='1g2'>1 Gb/s, 2nd card</option>
<option value='1g3'>1 Gb/s, 3rd card</option>
<option value='1g4'>1 Gb/s, 4rd card</option>
<option value='10g1'>10 Gb/s, 1st card</option>
<option value='10g2'>10 Gb/s, 2nd card</option>
<option value='10g3'>10 Gb/s, 3rd card</option>
<option value='10g4'>10 Gb/s, 4th card</option>
</select>";
            $out .= "<select name='iface2'>
<option value='none'>None</option>
<option value='eth0'>eth0</option>
<option value='eth1'>eth1</option>
<option value='eth2'>eth2</option>
<option value='eth3'>eth3</option>
<option value='eth4'>eth4</option>
<option value='eth5'>eth5</option>
<option value='10m1'>10 Mb/s, 1st card</option>
<option value='10m2'>10 Mb/s, 2nd card</option>
<option value='10m3'>10 Mb/s, 3rd card</option>
<option value='10m4'>10 Mb/s, 4th card</option>
<option value='100m1'>100 Mb/s, 1st card</option>
<option value='100m2'>100 Mb/s, 2nd card</option>
<option value='100m3'>100 Mb/s, 3rd card</option>
<option value='100m4'>100 Mb/s, 4th card</option>
<option value='1g1'>1 Gb/s, 1st card</option>
<option value='1g2'>1 Gb/s, 2nd card</option>
<option value='1g3'>1 Gb/s, 3rd card</option>
<option value='1g4'>1 Gb/s, 4rd card</option>
<option value='10g1'>10 Gb/s, 1st card</option>
<option value='10g2'>10 Gb/s, 2nd card</option>
<option value='10g3'>10 Gb/s, 3rd card</option>
<option value='10g4'>10 Gb/s, 4th card</option>
</select></td>";
            $out .= "<td><input type='submit' text='Add' value='Add'></form></td></tr>";
        }
        $out .= "</table>";

        $out .= "</td></table>";

        ////////////////////////////////////////////
        // Add and remove machines to the cluster //
        ////////////////////////////////////////////
        $out .= "<h2>Machines in the cluster</h2>";
        $rm = mysqli_query($con, "SELECT * FROM machines WHERE cluster='".$a["id"]."'");
        $nm = mysqli_num_rows($rm);
        $out .= "<table border='1'>
<tr><th>Hostname</th><th>IP(s)</th><th>Memory</th><th>HDD</th><th>Role</th><th>status</th><th>Action</th></tr>\n";
        for($j=0;$j<$nm;$j++){
            $am = mysqli_fetch_array($rm);

            $blockdevs = "";
            $r2 = mysqli_query($con, "SELECT * FROM blockdevices WHERE machine_id='".$am["id"]."'");
            $n2 = mysqli_num_rows($r2);
            for($i2=0; $i2<$n2; $i2++){
                $a2 = mysqli_fetch_array($r2);
                $blockdevs .= $a2["name"] . ": " . mb_to_smart_size($a2["size_mb"]) . "<br>";
            }

            $q_mip = "SELECT INET_NTOA(ip) AS addr FROM ips WHERE machine='".$am["id"]."' ORDER BY network";
            $r_mip = mysqli_query($con, $q_mip);
            $n_mip = mysqli_num_rows($r_mip);
            $my_addrs = "";
            for($k=0;$k<$n_mip;$k++){
                $a_mip = mysqli_fetch_array($r_mip);
                if($my_addrs == ""){
                    $my_addrs .= $a_mip["addr"];
                }else{
                    $my_addrs .= "<br>".$a_mip["addr"];
                }
            }

            $out .= "<tr>";
            $out .= "<td>".$am["hostname"]."</td>";
            $out .= "<td>$my_addrs</td>";
            $out .= "<td>". mb_to_smart_size($am["memory"]) ."</td>";
            $out .= "<td>$blockdevs</td>";
            $out .= "<td>".$am["role"]."</td>";
            $out .= "<td>".$am["status"]."</td>";
            $out .= "<td><form action='?'><input type='hidden' name='v-open-tab' value='$name'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='remove_from_cluster'><input type='hidden' name='machine-id' value='".$am["id"]."'><input type='hidden' name='cluster-id' value='$cluster_id'><input type='submit' text='Remove' value='Remove'></form></td>";
            $out .= "</tr>";
        }
        $out .= "</table>";
        $out .= "<h2>Available machines</h2>";
        $rm = mysqli_query($con, "SELECT * FROM machines WHERE cluster IS NULL");
        $nm = mysqli_num_rows($rm);
        $out .= "<form action='?'><input type='hidden' name='h-open-tab' value='Clusters'><input type='hidden' name='action' value='add_machines_to_cluster'><table border='1'>
<tr><th>Serial</th><th>Memory</th><th>HDD</th><th>status</th><th>Select</th></tr>\n";
        for($j=0;$j<$nm;$j++){
            $am = mysqli_fetch_array($rm);

            $blockdevs = "";
            $r2 = mysqli_query($con, "SELECT * FROM blockdevices WHERE machine_id='".$am["id"]."'");
            $n2 = mysqli_num_rows($r2);
            for($i2=0; $i2<$n2; $i2++){
                $a2 = mysqli_fetch_array($r2);
                $blockdevs .= $a2["name"] . ": " . mb_to_smart_size($a2["size_mb"]) . "<br>";
            }

            $out .= "<tr>";
            $out .= "<td>".$am["serial"]."</td>";
            $out .= "<td>". mb_to_smart_size($am["memory"]) ."</td>";
            $out .= "<td>$blockdevs</td>";
            $out .= "<td>".$am["status"]."</td>";
            $out .= "<td><input type='checkbox' name='id[]' value='".$am["id"]."'></td>";
            $out .= "</tr>";
        }
        $out .= "</table>";
        $out .= "<select name=\"role\">";
        $qr = "SELECT * FROM roles";
        $rr = mysqli_query($con, $qr);
        $nr = mysqli_num_rows($rr);
        for($j=0;$j<$nr;$j++){
            $a = mysqli_fetch_array($rr);
            $role_name = $a["name"];
            $out .= "<option value=\"$role_name\">$role_name</option>";
        }
        $out .= "</select>";
        $out .= "<input type='hidden' name='v-open-tab' value='$name'><input type='hidden' name='cluster-id' value='$cluster_id'><input type=\"submit\" text=\"Add to cluster\" value=\"Add to cluster\"></form>";
        $out .= "</div>\n";
    }
    return $out;
}

function role_list($con,$conf){
    $out = "";
    $out .= "<table border='1'>
<tr><th>Role name</th><th>Action</th></tr>\n";
    $r = mysqli_query($con, "SELECT * FROM roles");
    $n = mysqli_num_rows($r);
    for($i=0;$i<$n;$i++){
        $a = mysqli_fetch_array($r);
        $name = $a["name"];
        $id   = $a["id"];
        $out .= "<tr><td>$name</td><td><form action='?'><input type='hidden' name='h-open-tab' value='Roles'><input type='hidden' name='action' value='delete_role'><input type='hidden' name='id' value='$id'><input type=\"submit\" text=\"Delete\" value=\"Delete\"></form></td></tr>";
    }
    $out .= "<td><form action='?'><input type='text' name='name'></td><td><input type='hidden' name='h-open-tab' value='Roles'><input type='hidden' name='action' value='add_role'><input type=\"submit\" text=\"Add\" value=\"Add\"></form></td>";
    $out .= "</table>";
    return $out;
}

if( isset($_REQUEST["h-open-tab"])){
    switch($_REQUEST["h-open-tab"]){
    case "Clusters":
        $onload_h_tab = "ClustersBt";
        break;
    case "Roles":
        $onload_h_tab = "RolesBt";
        break;
    case "Networks":
        $onload_h_tab = "NetworksBt";
        break;
    default:
        $onload_h_tab = "MachinesBt";
        break;
    }
}else{
    $onload_h_tab = "MachinesBt";
}

if( isset($_REQUEST["v-open-tab"])){
    // This is to check there's no XSS here !
    if(validate_fqdn("v-open-tab") === FALSE){
        $onload_v_tab = "defaultOpenCluster";
    }else{
        $onload_v_tab = $_REQUEST["v-open-tab"]."Bt";
    }
}else{
    $onload_v_tab = "defaultOpenCluster";
}

function cidr_selector($cidr){
    $out = "";
    $out .= "<select name='cidr'>";
    for($i=8;$i<=32;$i++){
        if($i == $cidr){
            $selected = ' selected';
        }else{
            $selected = '';
        }
        $out .= "<option value='$i' $selected>/$i</option>";
    }
    $out .= "</select>";
    return $out;
}

function network_list($con,$conf){
    $out = "";
    $out .= "<table border='1'>
<tr><th>Network name</th><th>Network ip</th><th>CIDR</th><th>Is public</th><th>Action</th></tr>\n";
    $q = "SELECT * FROM networks ORDER BY ip";
    $r = mysqli_query($con, $q);
    $n = mysqli_num_rows($r);
    for($i=0;$i<$n;$i++){
        $a = mysqli_fetch_array($r);
        $id        = $a["id"];
        $name      = $a["name"];
        $ip        = $a["ip"];
        $cidr      = $a["cidr"];
        $is_public = $a["is_public"];
        if($is_public == "yes"){
            $public_selected = " checked";
        }else{
            $public_selected = "";
        }
    $out .= "<tr><form action='?'><td><input type='text' name='name' value='$name'></td><td><input type='text' name='ip' value='$ip'></td>
<td>".cidr_selector($cidr)."</td>
<td><input type='checkbox' name='is_public' value='yes'$public_selected></td>
<td><input type='hidden' name='id' value='$id'><input type='hidden' name='h-open-tab' value='Networks'><input type='hidden' name='action' value='edit_network'><input type=\"submit\" text=\"Save\" value=\"Save\"></form>
<form action='?'><input type='hidden' name='id' value='$id'><input type='hidden' name='h-open-tab' value='Networks'><input type='hidden' name='action' value='delete_network'><input type=\"submit\" text=\"Delete\" value=\"Delete\"></form>
</td></tr>";
    }
    $out .= "<tr><form action='?'><td><input type='text' name='name'></td><td><input type='text' name='ip'></td>
<td>".cidr_selector(24)."</td>
<td><input type='checkbox' name='is_public' value='yes'></td>
<td><input type='hidden' name='h-open-tab' value='Networks'><input type='hidden' name='action' value='create_network'><input type=\"submit\" text=\"Add\" value=\"Add\"></form></td></tr>";
    $out .= "</table>";
    return $out;
}

print("
<html><head><link rel='stylesheet' href='oci.css'></head>
<head>
<style>
body {font-family: Arial;}

/*************** horizontal tabs ***************/
/* Style the tab */
.tab {
    overflow: hidden;
    border: 1px solid #ccc;
    background-color: #f1f1f1;
}

/* Style the buttons inside the tab */
.tab button {
    background-color: inherit;
    float: left;
    border: none;
    outline: none;
    cursor: pointer;
    padding: 14px 16px;
    transition: 0.3s;
    font-size: 17px;
}

/* Change background color of buttons on hover */
.tab button:hover {
    background-color: #ddd;
}

/* Create an active/current tablink class */
.tab button.active {
    background-color: #ccc;
}

/* Style the tab content */
.tabcontent {
    display: none;
    padding: 6px 12px;
    border: 1px solid #ccc;
    border-top: none;
}
/*************** vertical tabs ***************/
/* Style the tab */
.vtab {
    float: left;
    border: 1px solid #ccc;
    background-color: #f1f1f1;
}

/* Style the buttons inside the tab */
.vtab button {
    display: block;
    background-color: inherit;
    color: black;
    padding: 22px 16px;
    width: 100%;
    border: none;
    outline: none;
    text-align: left;
    cursor: pointer;
    transition: 0.3s;
    font-size: 17px;
}

/* Change background color of buttons on hover */
.vtab button:hover {
    background-color: #ddd;
}

/* Create an active/current tab button class */
.vtab button.active {
    background-color: #ccc;
}

/* Style the tab content */
.vtabcontent {
    float: left;
    padding: 0px 12px;
    border: 1px solid #ccc;
    border-left: none;
    height: 300px;
}
</style>
<script>
function openTab(evt, tabName) {
    var i, tabcontent, tablinks;
    tabcontent = document.getElementsByClassName(\"tabcontent\");
    for (i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = \"none\";
    }
    tablinks = document.getElementsByClassName(\"tablinks\");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].className = tablinks[i].className.replace(\" active\", \"\");
    }
    document.getElementById(tabName).style.display = \"block\";
    evt.currentTarget.className += \" active\";
}


function openVTab(evt, clusterName) {
    var i, vtabcontent, vtablinks;
    vtabcontent = document.getElementsByClassName(\"vtabcontent\");
    for (i = 0; i < vtabcontent.length; i++) {
        vtabcontent[i].style.display = \"none\";
    }
    vtablinks = document.getElementsByClassName(\"vtablinks\");
    for (i = 0; i < vtablinks.length; i++) {
        vtablinks[i].className = vtablinks[i].className.replace(\" active\", \"\");
    }
    document.getElementById(clusterName).style.display = \"block\";
    evt.currentTarget.className += \" active\";
}

function myLoadEvent(){
    document.getElementById(\"$onload_h_tab\").click();
    document.getElementById(\"$onload_v_tab\").click();
}
</script>
</head>
<body onload='myLoadEvent()'>
<center><h1>OpenStack cluster Installer Web GUI</h1></center>
<div class=\"tab\">
  <button id=\"MachinesBt\" class=\"tablinks\" onclick=\"openTab(event, 'Machines')\">Machines</button>
  <button id=\"ClustersBt\" class=\"tablinks\" onclick=\"openTab(event, 'Clusters')\">Clusters</button>
  <button id=\"RolesBt\" class=\"tablinks\" onclick=\"openTab(event, 'Roles')\">Roles</button>
  <button id=\"NetworksBt\" class=\"tablinks\" onclick=\"openTab(event, 'Networks')\">Networks</button>
</div>

<div id=\"Machines\" class=\"tabcontent\">
    <center>");
    print(machine_list($con));
    print($out_message);
    print("
    </center>
</div>
<div id=\"Clusters\" class=\"tabcontent\">
    <div class=\"vtab\">
        <button class=\"vtablinks active\" onclick=\"openVTab(event, 'ClusterLifcycle')\" id=\"defaultOpenCluster\">Clusters Lifecycle</button>\n");
        print(clusterButtonList($con));
        print("
    </div>
    <div id=\"ClusterLifcycle\" class=\"vtabcontent\">");
        print(cluster_list($con,$conf));
        print("
    </div>");
    print(cluster_config_list($con,$conf));
    print("
</div>
<div id=\"Roles\" class=\"tabcontent\">
    <center>");
    print(role_list($con,$conf));
    print("
    </center>
</div>
<div id=\"Networks\" class=\"tabcontent\">");
print(network_list($con,$conf));
print("</div>
");

print("</body></html>");

?>
